import logging
import subprocess
from dataclasses import dataclass
from typing import Optional


@dataclass
class SuccessfulCommandResult:
    stdout: str
    stderr: str


class CommandNotFoundException(Exception):
    pass


class CommandErrorException(Exception):
    def __init__(self, message, return_code, stdout, stderr):
        super().__init__(message)

        self.return_code = return_code
        self.stdout = stdout
        self.stderr = stderr


def run_command(command_parts):
    if isinstance(command_parts, str):
        command_parts = command_parts.split()
    combined_command = " ".join(command_parts)
    logging.debug(f"Running command '{combined_command}'")
    try:
        result = subprocess.run(
            command_parts, stdout=subprocess.PIPE, stderr=subprocess.PIPE, input="", universal_newlines=True
        )
    except FileNotFoundError:
        logging.debug(f"Command not found for '{combined_command}'.")
        raise CommandNotFoundException(f"Command not found: {combined_command}")
    if result.returncode != 0:
        logging.debug(
            f"Non-zero exit code for '{combined_command}'\n"
            f"Exit code = {result.returncode}\n"
            f"Stdout = {result.stdout}\n"
            f"Stderr = {result.stderr}\n"
        )
        raise CommandErrorException(
            f"Command returned exit code {result.returncode}: {combined_command}",
            result.returncode,
            result.stdout,
            result.stderr,
        )
    logging.debug(f"Result of '{combined_command}'. Stdout = {result.stdout}. Stderr = {result.stderr}.")

    return SuccessfulCommandResult(result.stdout, result.stderr)


def can_run_command(command) -> bool:
    try:
        run_command(command)
        return True
    except Exception:
        logging.debug("Exception running command", exc_info=True)
        return False


def get_stdout(command) -> Optional[str]:
    try:
        return run_command(command).stdout
    except Exception:
        return None


def get_stderr(command) -> Optional[str]:
    try:
        return run_command(command).stderr
    except Exception:
        return None



from daktari.check import Check, CheckResult
from daktari.command_utils import can_run_command
from daktari.os import OS


class GoogleCloudSdkInstalled(Check):
    name = "google.cloudSdkInstalled"

    suggestions = {
        OS.OS_X: "<cmd>brew install --cask google-cloud-sdk</cmd>",
        OS.UBUNTU: "<cmd>sudo snap install google-cloud-sdk --classic</cmd>",
        OS.GENERIC: "Install gcloud: https://cloud.google.com/sdk/docs/quickstart",
    }

    def check(self) -> CheckResult:
        return self.verify(can_run_command("gcloud --version"), "Google Cloud SDK is <not/> installed")


class CloudSqlProxyInstalled(Check):
    name = "google.cloudSqlProxyInstalled"
    depends_on = [GoogleCloudSdkInstalled]

    suggestions = {
        OS.GENERIC: "Install Cloud SQL Proxy: <cmd>gcloud components install cloud_sql_proxy</cmd>",
    }

    def check(self) -> CheckResult:
        return self.verify(can_run_command("cloud_sql_proxy --version"), "Cloud SQL Proxy is <not/> installed")



import abc
import re
from dataclasses import dataclass
from enum import Enum
from typing import Dict, List, Optional, Type

from daktari.command_utils import can_run_command


class CheckStatus(Enum):
    PASS = "PASS"
    FAIL = "FAIL"


@dataclass
class CheckResult:
    name: str
    status: CheckStatus
    summary: str
    suggestions: Dict[str, str]


class Check:
    name: str
    depends_on: List[Type] = []
    suggestions: Dict[str, str] = {}
    run_on: Optional[str] = None

    def passed(self, message: str) -> CheckResult:
        return CheckResult(self.name, CheckStatus.PASS, message, self.suggestions)

    def failed(self, message: str) -> CheckResult:
        return CheckResult(self.name, CheckStatus.FAIL, message, self.suggestions)

    def verify(self, passed: bool, dualMessage: str) -> CheckResult:
        pattern = re.compile(" <not/> ")
        if passed:
            return self.passed(pattern.sub(" ", dualMessage))
        else:
            return self.failed(pattern.sub(" not ", dualMessage))

    def verify_install(self, program: str, version_flag: str = "--version") -> CheckResult:
        return self.verify(can_run_command(f"{program} {version_flag}"), f"{program} is <not/> installed")

    @abc.abstractmethod
    def check(self) -> CheckResult:
        raise NotImplementedError("check must be implemented")

    def override_suggestions(self, suggestions: Dict[str, str]) -> "Check":
        self.suggestions = suggestions
        return self

    def suggest(self, os: str, text: str) -> "Check":
        return self.override_suggestions({os: text})

    def only_on(self, os: str) -> "Check":
        self.run_on = os
        return self



from daktari.command_utils import get_stdout


def is_ascii(path: str) -> bool:
    file_output = get_stdout(["file", path]) or ""
    parts = file_output.strip().split(": ")
    return parts[1] == "ASCII text"



from daktari.check import Check, CheckResult
from daktari.os import OS


class WatchmanInstalled(Check):
    name = "watchman.installed"

    suggestions = {
        OS.OS_X: "<cmd>brew install watchman</cmd>",
        OS.GENERIC: "Install watchman: https://facebook.github.io/watchman/docs/install.html#buildinstall",
    }

    def check(self) -> CheckResult:
        return self.verify_install("watchman")


class MkcertInstalled(Check):
    name = "mkcert.installed"

    suggestions = {
        OS.OS_X: "<cmd>brew install mkcert</cmd>",
        OS.GENERIC: "Install mkcert: https://mkcert.dev/#installation",
    }

    def check(self) -> CheckResult:
        return self.verify_install("mkcert")


class KtlintInstalled(Check):
    name = "ktlint.installed"

    suggestions = {
        OS.OS_X: "<cmd>brew install ktlint</cmd>",
        OS.GENERIC: "Install ktlint: https://github.com/pinterest/ktlint#installation",
    }

    def check(self) -> CheckResult:
        return self.verify_install("ktlint")


class JqInstalled(Check):
    name = "jq.installed"

    suggestions = {
        OS.OS_X: "<cmd>brew install jq</cmd>",
        OS.UBUNTU: "<cmd>sudo apt install jq</cmd>",
        OS.GENERIC: "Install jq: https://stedolan.github.io/jq/download/"
    }

    def check(self) -> CheckResult:
        return self.verify_install("jq")


class FlywayInstalled(Check):
    name = "flyway.installed"

    suggestions = {
        OS.OS_X: "<cmd>brew install flyway</cmd>",
        OS.UBUNTU: "<cmd>snap install flyway</cmd>",
        OS.GENERIC: "Install flyway: https://flywaydb.org/documentation/usage/commandline/#download-and-installation"
    }

    def check(self) -> CheckResult:
        return self.verify_install("flyway", "-v")



from dataclasses import dataclass
from pathlib import Path
from typing import Any, Dict, List, Optional

from daktari.check import Check


@dataclass
class Config:
    title: Optional[str]
    checks: List[Check]


def read_config(config_path: Path) -> Config:
    variables: Dict[str, Any] = {}
    exec(config_path.read_text(), variables)
    checks = variables.get("checks", [])
    title = variables.get("title", None)
    return Config(title, checks)



import logging
import re
from typing import Optional

from daktari.check import Check, CheckResult
from daktari.command_utils import get_stderr
from daktari.os import OS

java_version_pattern = re.compile('^.*version "(.*?)".*$', re.MULTILINE)
one_dot_pattern = re.compile("1\\.([0-9]+)")
other_pattern = re.compile("([0-9]+)")


def get_java_version() -> Optional[int]:
    version_output = get_stderr("java -version")
    if version_output:
        match = java_version_pattern.search(version_output)
        if match:
            version_string = match.group(1)
            logging.debug(f"Java version string: {version_string}")
            return get_java_number(version_string)
    return None


def get_java_number(version_string: str) -> Optional[int]:
    one_dot_match = one_dot_pattern.search(version_string)
    if one_dot_match:
        return int(one_dot_match.group(1))
    other_match = other_pattern.search(version_string)
    if other_match:
        return int(other_match.group(1))
    return None


class JavaVersion(Check):
    name = "java.version"

    def __init__(self, required_version: int):
        self.required_version = required_version

    suggestions = {
        OS.GENERIC: "Install Java",
    }

    def check(self) -> CheckResult:
        java_version = get_java_version()
        logging.info(f"Java version: {java_version}")
        return self.verify(java_version == self.required_version, f"Java {self.required_version} is <not/> installed")
